/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2011 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#include "headers.h"
#include "forward.h"
#include "views3.h"

#include "palette.h"
#include "widgets.h"
#include "controller.h"

namespace Crow {

void RegisterViews3() {

	Palette * palette= Palette::get();

	// -----------------------------------

	palette->takeEntry(new TObjectEntry<GtkEntryCompletionView>("GtkEntryCompletion", GTK_TYPE_ENTRY_COMPLETION));

	palette->takeEntry(new TWidgetEntry<GtkEntryView>("GtkEntry", GTK_TYPE_ENTRY, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkComboBoxView>("GtkComboBox", GTK_TYPE_COMBO_BOX, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<CrowComboBoxEntryView>("CrowComboBoxEntry", GTK_TYPE_COMBO_BOX, TAB_STANDARD));

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkSpinButtonUpdatePolicy")).
		addValue("GTK_UPDATE_ALWAYS", Gtk::UPDATE_ALWAYS).
		addValue("GTK_UPDATE_IF_VALID", Gtk::UPDATE_IF_VALID);

	palette->takeEntry(new TWidgetEntry<GtkSpinButtonView>("GtkSpinButton", GTK_TYPE_SPIN_BUTTON, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkSensitivityType")).
		addValue("GTK_SENSITIVITY_AUTO", Gtk::SENSITIVITY_AUTO).
		addValue("GTK_SENSITIVITY_ON", Gtk::SENSITIVITY_ON).
		addValue("GTK_SENSITIVITY_OFF", Gtk::SENSITIVITY_OFF);

	palette->takeEntry(new TObjectEntry<ScaleMarkEntryView>("CrowScaleMarkEntry", G_TYPE_INVALID));

	palette->takeEntry(new TWidgetEntry<GtkHScaleView>("GtkHScale", GTK_TYPE_HSCALE, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkVScaleView>("GtkVScale", GTK_TYPE_VSCALE, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkProgressBarView>("GtkProgressBar", GTK_TYPE_PROGRESS_BAR, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkIconSize")).
		addValue("GTK_ICON_SIZE_MENU", Gtk::ICON_SIZE_MENU).
		addValue("GTK_ICON_SIZE_SMALL_TOOLBAR", Gtk::ICON_SIZE_SMALL_TOOLBAR).
		addValue("GTK_ICON_SIZE_LARGE_TOOLBAR", Gtk::ICON_SIZE_LARGE_TOOLBAR).
		addValue("GTK_ICON_SIZE_BUTTON", Gtk::ICON_SIZE_BUTTON).
		addValue("GTK_ICON_SIZE_DND", Gtk::ICON_SIZE_DND).
		addValue("GTK_ICON_SIZE_DIALOG", Gtk::ICON_SIZE_DIALOG);

	palette->takeEntry(new TWidgetEntry<GtkImageView>("GtkImage", GTK_TYPE_IMAGE, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkHSeparatorView>("GtkHSeparator", GTK_TYPE_HSEPARATOR, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkVSeparatorView>("GtkVSeparator", GTK_TYPE_VSEPARATOR, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkSizeGroupMode")).
		addValue("GTK_SIZE_GROUP_NONE", Gtk::SIZE_GROUP_NONE).
		addValue("GTK_SIZE_GROUP_HORIZONTAL", Gtk::SIZE_GROUP_HORIZONTAL).
		addValue("GTK_SIZE_GROUP_VERTICAL", Gtk::SIZE_GROUP_VERTICAL).
		addValue("GTK_SIZE_GROUP_BOTH", Gtk::SIZE_GROUP_BOTH);

	palette->takeEntry(new TObjectEntry<GtkSizeGroupView>("GtkSizeGroup", GTK_TYPE_SIZE_GROUP)).
		setPaletteTab(TAB_ADDITIONAL).
		setCanvasEditor("canvas|GlibObject");

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkWrapMode")).
		addValue("GTK_WRAP_NONE", Gtk::WRAP_NONE).
		addValue("GTK_WRAP_CHAR", Gtk::WRAP_CHAR).
		addValue("GTK_WRAP_WORD", Gtk::WRAP_WORD).
		addValue("GTK_WRAP_WORD_CHAR", Gtk::WRAP_WORD_CHAR);

	palette->takeEntry(new TWidgetEntry<GtkTextViewView>("GtkTextView", GTK_TYPE_TEXT_VIEW, TAB_VIEWS));

	palette->takeEntry(new EnumEntry("GtkSelectionMode")).
		addValue("GTK_SELECTION_NONE", Gtk::SELECTION_NONE).
		addValue("GTK_SELECTION_SINGLE", Gtk::SELECTION_SINGLE).
		addValue("GTK_SELECTION_BROWSE", Gtk::SELECTION_BROWSE).
		addValue("GTK_SELECTION_MULTIPLE", Gtk::SELECTION_MULTIPLE);

	palette->takeEntry(new TWidgetEntry<GtkIconViewView>("GtkIconView", GTK_TYPE_ICON_VIEW, TAB_VIEWS));

	palette->takeEntry(new TWidgetEntry<GtkCellViewView>("GtkCellView", GTK_TYPE_CELL_VIEW, TAB_VIEWS));

	palette->takeEntry(new EnumEntry("GtkTreeViewGridLines")).
		addValue("GTK_TREE_VIEW_GRID_LINES_NONE", Gtk::TREE_VIEW_GRID_LINES_NONE).
		addValue("GTK_TREE_VIEW_GRID_LINES_HORIZONTAL", Gtk::TREE_VIEW_GRID_LINES_HORIZONTAL).
		addValue("GTK_TREE_VIEW_GRID_LINES_VERTICAL", Gtk::TREE_VIEW_GRID_LINES_VERTICAL).
		addValue("GTK_TREE_VIEW_GRID_LINES_BOTH", Gtk::TREE_VIEW_GRID_LINES_BOTH);

	palette->takeEntry(new TWidgetEntry<GtkTreeViewView>("GtkTreeView", GTK_TYPE_TREE_VIEW, TAB_VIEWS));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<GtkColorButtonView>("GtkColorButton", GTK_TYPE_COLOR_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkColorSelectionView>("GtkColorSelection", GTK_TYPE_COLOR_SELECTION, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserButtonView>("GtkFileChooserButton", GTK_TYPE_FILE_CHOOSER_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserWidgetView>("GtkFileChooserWidget", GTK_TYPE_FILE_CHOOSER_WIDGET, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFontButtonView>("GtkFontButton", GTK_TYPE_FONT_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFontSelectionView>("GtkFontSelection", GTK_TYPE_FONT_SELECTION, TAB_SELECTORS));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<GtkFixedView>("GtkFixed", GTK_TYPE_FIXED, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkLayoutView>("GtkLayout", GTK_TYPE_LAYOUT, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkHScrollbarView>("GtkHScrollbar", GTK_TYPE_HSCROLLBAR, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkVScrollbarView>("GtkVScrollbar", GTK_TYPE_VSCROLLBAR, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkEventBoxView>("GtkEventBox", GTK_TYPE_EVENT_BOX, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkDrawingAreaView>("GtkDrawingArea", GTK_TYPE_DRAWING_AREA, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkCalendarView>("GtkCalendar", GTK_TYPE_CALENDAR, TAB_SPECIAL));

	palette->takeEntry(new EnumEntry("GtkArrowType")).
		addValue("GTK_ARROW_UP", Gtk::ARROW_UP).
		addValue("GTK_ARROW_DOWN", Gtk::ARROW_DOWN).
		addValue("GTK_ARROW_LEFT", Gtk::ARROW_LEFT).
		addValue("GTK_ARROW_RIGHT", Gtk::ARROW_RIGHT);

	palette->takeEntry(new TWidgetEntry<GtkArrowView>("GtkArrow", GTK_TYPE_ARROW, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<CustomView>("CrowCustom", GTK_TYPE_WIDGET, TAB_SPECIAL));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<DebugButtonView>("CrowDebugButton", GTK_TYPE_WIDGET, TAB_DEBUG)).
		setFlags(efDebug);

	palette->takeEntry(new TWidgetEntry<DebugWindowView>("CrowDebugWindow", GTK_TYPE_WINDOW, TAB_DEBUG)).
		setFlags(efDebug);

	// -----------------------------------

	palette->takeEntry(new TObjectEntry<GtkStatusIconView>("GtkStatusIcon", GTK_TYPE_STATUS_ICON)).
		setPaletteTab(TAB_SPECIAL).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TWidgetEntry<GtkLinkButtonView>("GtkLinkButton", GTK_TYPE_LINK_BUTTON, TAB_ADDITIONAL));

	// -----------------------------------

	palette->takeEntry(new TObjectEntry<GtkRecentFilterView>("GtkRecentFilter", GTK_TYPE_RECENT_FILTER));

	palette->takeEntry(new EnumEntry("GtkRecentSortType")).
		addValue("GTK_RECENT_SORT_NONE", Gtk::RECENT_SORT_NONE).
		addValue("GTK_RECENT_SORT_MRU", Gtk::RECENT_SORT_MRU).
		addValue("GTK_RECENT_SORT_LRU", Gtk::RECENT_SORT_LRU).
		addValue("GTK_RECENT_SORT_CUSTOM", Gtk::RECENT_SORT_CUSTOM);

	palette->takeEntry(new TWidgetEntry<GtkRecentChooserWidgetView>("GtkRecentChooserWidget", GTK_TYPE_RECENT_CHOOSER_WIDGET, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkRecentChooserDialogView>("GtkRecentChooserDialog", GTK_TYPE_RECENT_CHOOSER_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new TObjectEntry<GtkRecentActionView>("GtkRecentAction", GTK_TYPE_RECENT_ACTION));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<GtkSpinnerView>("GtkSpinner", GTK_TYPE_SPINNER, TAB_SPECIAL));

}

// GtkEntryCompletionView ------------------------------------------------------

GtkEntryCompletionView::GtkEntryCompletionView() {
	addSignalsProperty();
	addProperty("inline-completion", prScalar, "bool", CAny::createBool(false));
	addProperty("minimum-key-length", prScalar, "int", CAny::createInt(1));
	addProperty("popup-completion", prScalar, "bool", CAny::createBool(true));
	addInertProperty("strings", prVector, "string").
		setFlag(sfTranslatable).
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("text")
		));
	addProperty("popup-set-width", prScalar, "bool", CAny::createBool(true));
	addProperty("popup-single-match", prScalar, "bool", CAny::createBool(true));
	addProperty("inline-selection", prScalar, "bool", CAny::createBool(false));
}

Any GtkEntryCompletionView::createInstance() {
	return CAny::createObject(getType(), Gtk::EntryCompletion::create());
}

// GtkEntryView ----------------------------------------------------------------

GtkEntryView::GtkEntryView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("activates-default", prScalar, "bool", CAny::createBool(false));
	addProperty("editable", prScalar, "bool", CAny::createBool(true));
	addProperty("has-frame", prScalar, "bool", CAny::createBool(true));
	addProperty("invisible-char-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setInvisibleCharSet));
	addProperty("invisible-char", prScalar, "char").
		setValueGet(sigc::mem_fun(*this, &GtkEntryView::getInvisibleChar)).
		setFlag(sfReadOnly);
	addProperty("max-length", prScalar, "int", CAny::createInt(0));
	addProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("visibility", prScalar, "bool", CAny::createBool(true));
	addProperty("width-chars", prScalar, "int", CAny::createInt(-1));
	addProperty("xalign", prScalar, "float", CAny::createFloat(0));
	addProperty("completion", prEntity, "GtkEntryCompletion").
		setValueGet(sigc::mem_fun(*this, &GtkEntryView::getCompletion)).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setCompletion));
	addInertProperty("inner-border-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setInnerBorderSet));
	addInertProperty("inner-border", prScalar, "GtkBorder", CAny::create("GtkBorder", Border())).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setInnerBorder)).
		setFlag(sfReadOnly);
	addProperty("truncate-multiline", prScalar, "bool", CAny::createBool(false));
	addProperty("shadow-type", prScalar, "GtkShadowType", CAny::createEnum("GtkShadowType", Gtk::SHADOW_IN)).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setShadowType));
	addProperty("caps-lock-warning", prScalar, "bool", CAny::createBool(true));
	addProperty("overwrite-mode", prScalar, "bool", CAny::createBool(false));
	addProperty("progress-fraction", prScalar, "double", CAny::createDouble(0));
	addProperty("progress-pulse-step", prScalar, "double", CAny::createDouble(0.1));
	addProperty("primary-icon-name", prScalar, "string", CAny::createString("")).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setPrimaryIconName)).
		setEditor("icon-name|string");
	addInertProperty("primary-icon-tooltip", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("primary-icon-sensitive", prScalar, "bool", CAny::createBool(true));
	addInertProperty("primary-icon-activatable", prScalar, "bool", CAny::createBool(true));
	addProperty("secondary-icon-name", prScalar, "string", CAny::createString("")).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setSecondaryIconName)).
		setEditor("icon-name|string");
	addInertProperty("secondary-icon-tooltip", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("secondary-icon-sensitive", prScalar, "bool", CAny::createBool(true));
	addInertProperty("secondary-icon-activatable", prScalar, "bool", CAny::createBool(true));
}

void GtkEntryView::initDesign() {
	getObject()->set_text(getDesignLabel());
}

void GtkEntryView::setInvisibleCharSet(Property * property, Any value) {
	bool set= value->getBool();
	if(set)
		findProperty("invisible-char")->clearFlag(sfReadOnly);
	else
		findProperty("invisible-char")->setFlag(sfReadOnly);
	getObject()->property_invisible_char_set()= set;
	touch("invisible-char");
}

Any GtkEntryView::getInvisibleChar(Property * property) {
	if(getObject()->property_invisible_char_set())
		return CAny::create<UniChar>("char", getObject()->get_invisible_char());
	else
		return CAny::create<UniChar>("char", '*');
}

Any GtkEntryView::getCompletion(Property * property) {
	return CAny::createObject(getObject()->get_completion());
}

void GtkEntryView::setCompletion(Property * property, Any value) {
	getObject()->set_completion(value->getObject<Gtk::EntryCompletion>());
}

void GtkEntryView::setInnerBorderSet(Property * property, Any value) {
	property->setInert(value);
	Property * P= findProperty("inner-border");
	if(value->getBool()) {
		P->clearFlag(sfReadOnly);
		setInnerBorder(P, P->getDefault());
	} else {
		P->setFlag(sfReadOnly);
		P->setInert(P->getDefault());
		gtk_entry_set_inner_border(getObject()->gobj(), NULL);
	}
	touch("inner-border");
}

void GtkEntryView::setInnerBorder(Property * property, Any value) {
	property->setInert(value);
	Gtk::Border border= *value->get<Border>();
	getObject()->set_inner_border(border);
}

void GtkEntryView::setShadowType(Property * property, Any value) {
	getObject()->property_shadow_type()= static_cast<Gtk::ShadowType>(value->getEnum());
	getObject()->queue_draw();
}

void GtkEntryView::setPrimaryIconName(Property * property, Any value) {
	ustring_cr name= *value->getString();
	gtk_entry_set_icon_from_icon_name(
		getObject()->gobj(), GTK_ENTRY_ICON_PRIMARY, name.empty() ? NULL : name.c_str());
}

void GtkEntryView::setSecondaryIconName(Property * property, Any value) {
	ustring_cr name= *value->getString();
	gtk_entry_set_icon_from_icon_name(
		getObject()->gobj(), GTK_ENTRY_ICON_SECONDARY, name.empty() ? NULL : name.c_str());
}

// GtkComboBoxView -------------------------------------------------------------

GtkComboBoxView::GtkComboBoxView() {
	addInertProperty("add-tearoffs", prScalar, "bool", CAny::createBool(false));
	addProperty("focus-on-click", prScalar, "bool", CAny::createBool(true));
	addInertProperty("text-mode", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxView::setTextMode));
	addInertProperty("strings", prVector, "string").
		setFlag(sfReadOnly | sfTranslatable).
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxView::setStrings)).
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GlibObjectView::insertScalar),
			CAny::createString("text")
		));
	addProperty("active", prScalar, "int", CAny::createInt(-1));
	addProperty("tearoff-title", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("button-sensitivity", prScalar, "GtkSensitivityType", CAny::createEnum("GtkSensitivityType", Gtk::SENSITIVITY_AUTO));
}

Any GtkComboBoxView::createInstance() {
	Gtk::Widget * widget= new Gtk::ComboBoxText();
	SetData(widget, "CrowTypeHint", "GtkComboBox");
	return CAny::createObject(getType(), MakeRefPtr(widget));
}

void GtkComboBoxView::setTextMode(Property * property, Any value) {
	property->setInert(value);
	if(value->getBool()) {
		findProperty("strings")->clearFlag(sfReadOnly);
	} else {
		clearVectorProperty("strings")->setFlag(sfReadOnly);
		clearStrings();
	}
	touch("strings");
}

void GtkComboBoxView::setStrings(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::ComboBoxText> combo= getObject<Gtk::ComboBoxText>();
	int active= combo->get_active_row_number();
	combo->remove_all();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		ustring_cr str= *vec[i]->getString();
		combo->append(str);
	}
	combo->set_active(active);
}

void GtkComboBoxView::clearStrings() {
	getObject<Gtk::ComboBoxText>()->remove_all();
}

// CrowComboBoxEntryView -------------------------------------------------------

CrowComboBoxEntryView::CrowComboBoxEntryView() {
	addProperty("has-frame", prScalar, "bool", CAny::createBool(true));
	addProperty("text", prScalar, "string", CAny::createString("")).
		setValueGet(sigc::mem_fun(*this, &CrowComboBoxEntryView::getText)).
		setValueSet(sigc::mem_fun(*this, &CrowComboBoxEntryView::setText));
	addProperty("entry", prEntity, "GtkEntry").
		setValueGet(sigc::mem_fun(*this, &CrowComboBoxEntryView::getEntry)).
		setViewConfigure(sigc::mem_fun(*this, &CrowComboBoxEntryView::configureEntry)).
		setFlag(sfReadOnly);
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &CrowComboBoxEntryView::setActive));
}

Any CrowComboBoxEntryView::createInstance() {
	Gtk::Widget * widget= Glib::wrap(gtk_combo_box_text_new_with_entry());
	SetData(widget, "CrowTypeHint", "CrowComboBoxEntry");
	return CAny::createObject(getType(), MakeRefPtr(widget));
}

Any CrowComboBoxEntryView::getEntry(Property * property) {
	return CAny::createObject(MakeRefPtr(getObject<Gtk::ComboBox>()->get_entry()));
}

Any CrowComboBoxEntryView::getText(Property * property) {
	return CAny::createString(getObject<Gtk::ComboBox>()->get_entry()->get_text());
}

void CrowComboBoxEntryView::setText(Property * property, Any value) {
	getObject<Gtk::ComboBox>()->get_entry()->set_text(*value->getString());
}

void CrowComboBoxEntryView::setActive(Property * property, Any value) {
	Gtk::ComboBox * combo= GetPtr(getObject<Gtk::ComboBox>());
	int active= combo->get_active_row_number();
	combo->set_active(value->getInt());
	activeChanged(active);
}

void CrowComboBoxEntryView::configureEntry(Property * property, PEntityView view) {
	view->findProperty("has-frame")->setFlag(sfReadOnly | sfHidden);
	view->findProperty("text")->setFlag(sfReadOnly | sfHidden);
}

void CrowComboBoxEntryView::setStrings(Property * property, Any value) {
	property->setInert(value);
	Gtk::ComboBoxText * combo= GetPtr(getObject<Gtk::ComboBoxText>());
	int active= combo->get_active_row_number();
	combo->remove_all();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		ustring_cr str= *vec[i]->getString();
		combo->append(str);
	}
	combo->set_active(active);
	activeChanged(active);
}

void CrowComboBoxEntryView::clearStrings() {
	Gtk::ComboBoxText * combo= GetPtr(getObject<Gtk::ComboBoxText>());
	int active= combo->get_active_row_number();
	combo->remove_all();
	combo->set_active(-1);
	activeChanged(active);
}

void CrowComboBoxEntryView::activeChanged(int old) {
	Gtk::ComboBox * combo= GetPtr(getObject<Gtk::ComboBox>());
	int actual= combo->get_active_row_number();
	if(actual != old) {
		if(actual > -1)
			findProperty("text")->setFlag(sfReadOnly);
		else {
			combo->get_entry()->set_text("");
			findProperty("text")->clearFlag(sfReadOnly);
		}
		touch("text");
	}
}

// GtkSpinButtonView -----------------------------------------------------------

GtkSpinButtonView::GtkSpinButtonView() {
	findProperty("text")->setFlag(sfReadOnly | sfHidden);
	addProperty("adjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly).
		setViewConfigure(sigc::mem_fun(*this, &GtkSpinButtonView::configureAdjustment));
	addProperty("climb-rate", prScalar, "double", CAny::createDouble(0));
	addProperty("digits", prScalar, "unsigned", CAny::createUnsigned(0));
	addProperty("numeric", prScalar, "bool", CAny::createBool(false));
	addProperty("snap-to-ticks", prScalar, "bool", CAny::createBool(false));
	addProperty("update-policy", prScalar, "GtkSpinButtonUpdatePolicy", CAny::createEnum("GtkSpinButtonUpdatePolicy", Gtk::UPDATE_ALWAYS));
	addProperty("value", prScalar, "double", CAny::createDouble(0));
	addProperty("wrap", prScalar, "bool", CAny::createBool(false));
}

Any GtkSpinButtonView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr<Gtk::SpinButton>());
}

void GtkSpinButtonView::configureAdjustment(Property * property, PEntityView view) {
	view->findProperty("value")->setFlag(sfReadOnly);
}

// GtkRangeView ----------------------------------------------------------------

GtkRangeView::GtkRangeView() {
	addOrientationProperty();
	addProperty("inverted", prScalar, "bool", CAny::createBool(false));
	addProperty("show-fill-level", prScalar, "bool", CAny::createBool(false));
	addInertProperty("restrict-to-fill-level", prScalar, "bool", CAny::createBool(false));
	addProperty("fill-level", prScalar, "double", CAny::createDouble(0));
	addProperty("adjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly);
	addProperty("lower-stepper-sensitivity", prScalar, "GtkSensitivityType", CAny::createEnum("GtkSensitivityType", Gtk::SENSITIVITY_AUTO));
	addProperty("upper-stepper-sensitivity", prScalar, "GtkSensitivityType", CAny::createEnum("GtkSensitivityType", Gtk::SENSITIVITY_AUTO));
}

Slider GtkRangeView::getSlider(Manipulator manip) {
	Slider slider;
	if(
		(manip==SliderHor && (getObject<Gtk::HScale>() || getObject<Gtk::HScrollbar>())) ||
		(manip==SliderVer && (getObject<Gtk::VScale>() || getObject<Gtk::VScrollbar>()))
	) {
		slider.manip= manip;
		PGtkAdjustment adj= getObject<Gtk::Range>()->get_adjustment();
		slider.value= GetSliderValue(adj);
	}
	return slider;
}

void GtkRangeView::setSlider(Manipulator manip, double value) {
	PGtkAdjustment adj= getObject<Gtk::Range>()->get_adjustment();
	SetSliderValue(adj, value);
}

void GtkRangeView::initInstance() {
	GtkWidgetView::initInstance();
	getObject<Gtk::Range>()->set_fill_level(0);
	getObject<Gtk::Range>()->set_restrict_to_fill_level(false);
}

// ScaleMarkEntryView ----------------------------------------------------------

ScaleMarkEntry::ScaleMarkEntry() {
	SetData(this, "CrowTypeHint", "CrowScaleMarkEntry");
	value= 0;
	position= Gtk::POS_BOTTOM;
}

ScaleMarkEntryView::ScaleMarkEntryView() {
	addProperty("value", prScalar, "double", CAny::createDouble(0)).
		setValueGet(sigc::mem_fun(*this, &ScaleMarkEntryView::getValue)).
		setValueSet(sigc::mem_fun(*this, &ScaleMarkEntryView::setValue));
	addProperty("position", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_BOTTOM)).
		setValueGet(sigc::mem_fun(*this, &ScaleMarkEntryView::getPosition)).
		setValueSet(sigc::mem_fun(*this, &ScaleMarkEntryView::setPosition));
	addProperty("caption", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &ScaleMarkEntryView::getCaption)).
		setValueSet(sigc::mem_fun(*this, &ScaleMarkEntryView::setCaption));
}

Any ScaleMarkEntryView::getValue(Property * property) {
	return CAny::createDouble(getObject()->value);
}

void ScaleMarkEntryView::setValue(Property * property, Any value) {
	getObject()->value= value->getDouble();
}

Any ScaleMarkEntryView::getPosition(Property * property) {
	return CAny::createEnum("GtkPositionType", getObject()->position);
}

void ScaleMarkEntryView::setPosition(Property * property, Any value) {
	getObject()->position= static_cast<Gtk::PositionType>(value->getEnum());
}

Any ScaleMarkEntryView::getCaption(Property * property) {
	return CAny::createString(getObject()->caption);
}

void ScaleMarkEntryView::setCaption(Property * property, Any value) {
	getObject()->caption= *value->getString();
}

// GtkScaleView ----------------------------------------------------------------

GtkScaleView::GtkScaleView() {
	addProperty("digits", prScalar, "int", CAny::createInt(1));
	addProperty("draw-value", prScalar, "bool", CAny::createBool(true));
	addProperty("value-pos", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_TOP));
	addCanFocusProperty(true);
	addInertProperty("marks", prVector, "CrowScaleMarkEntry").
		setFlag(sfOverwrite).
		setValueSet(sigc::mem_fun(*this, &GtkScaleView::setMarks)).
		setElementInsert(sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GtkScaleView::insertEntity)).
		setLabelGet(sigc::mem_fun(*this, &GtkScaleView::getMarkLabel));
}

void GtkScaleView::setMarks(Property * property, Any value) {
	property->setInert(value);
	clearMarks();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PScaleMarkEntry entry= vec[i]->getObject<ScaleMarkEntry>();
		if(entry)
			addMark(entry);
	}
}

void GtkScaleView::clearMarks() {
	getObject<Gtk::Scale>()->clear_marks();
}

void GtkScaleView::addMark(PScaleMarkEntry entry) {
	getObject<Gtk::Scale>()->add_mark(entry->value, entry->position, entry->caption);
}

std::string GtkScaleView::getMarkLabel(Property * property, PNode node) {
	if(PNode value= getModel()->find(node, "value"))
		return value->getValue()->toString();
	else
		return *node->getName();
}

// GtkProgressBarView ----------------------------------------------------------

GtkProgressBarView::GtkProgressBarView() {
	addEventProperties();
	addOrientationProperty();
	addProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("fraction", prScalar, "double", CAny::createDouble(0));
	addProperty("inverted", prScalar, "bool", CAny::createBool(false));
	addProperty("pulse-step", prScalar, "double", CAny::createDouble(0.1));
	addProperty("show-text", prScalar, "bool", CAny::createBool(false));
	addProperty("ellipsize", prScalar, "PangoEllipsizeMode", CAny::createEnum("PangoEllipsizeMode", Pango::ELLIPSIZE_NONE));
}

// GtkImageView ----------------------------------------------------------------

GtkImageView::GtkImageView() {
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addProperty("icon-size", prScalar, "GtkIconSize", CAny::createEnum("GtkIconSize", Gtk::ICON_SIZE_BUTTON));
	addProperty("pixel-size", prScalar, "int", CAny::createInt(-1));
}

void GtkImageView::initInstance() {
	GtkMiscView::initInstance();
	getObject()->set_from_icon_name("", Gtk::ICON_SIZE_BUTTON);
}

// GtkSeparatorView ------------------------------------------------------------

GtkSeparatorView::GtkSeparatorView() {
	addOrientationProperty();
}

// GtkSizeGroupView ------------------------------------------------------------

GtkSizeGroupView::GtkSizeGroupView() {
	addSignalsProperty();
	addProperty("mode", prScalar, "GtkSizeGroupMode", CAny::createEnum("GtkSizeGroupMode", Gtk::SIZE_GROUP_HORIZONTAL));
	addProperty("ignore-hidden", prScalar, "bool", CAny::createBool(false));
	addInertProperty("widgets", prVector, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink).
		setValueSet(sigc::mem_fun(*this, &GtkSizeGroupView::setWidgets)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
}

Any GtkSizeGroupView::createInstance() {
	return CAny::createObject(getType(), Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL));
}

void GtkSizeGroupView::doneInstance() {
	clearWidgets();
	GlibObjectView::doneInstance();
}

void GtkSizeGroupView::clearWidgets() {
	Glib::RefPtr<Gtk::SizeGroup> sizeGroup= getObject<Gtk::SizeGroup>();
	for(;;) {
		GSList * list= gtk_size_group_get_widgets(sizeGroup->gobj());
		if(list) {
			GtkWidget * widget= GTK_WIDGET(list->data);
			gtk_size_group_remove_widget(sizeGroup->gobj(), widget);
		} else
			break;
	}
}

void GtkSizeGroupView::setWidgets(Property * property, Any value) {
	property->setInert(value);
	clearWidgets();
	Glib::RefPtr<Gtk::SizeGroup> sizeGroup= getObject<Gtk::SizeGroup>();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PGtkWidget obj= vec[i]->getObject<Gtk::Widget>();
		if(obj)
			sizeGroup->add_widget(*GetPtr(obj));
	}
}

// GtkTextViewView -------------------------------------------------------------

GtkTextViewView::GtkTextViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("accepts-tab", prScalar, "bool", CAny::createBool(true));
	addProperty("cursor-visible", prScalar, "bool", CAny::createBool(true));
	addProperty("editable", prScalar, "bool", CAny::createBool(true));
	addProperty("indent", prScalar, "int", CAny::createInt(0));
	addProperty("justification", prScalar, "GtkJustification", CAny::createEnum("GtkJustification", Gtk::JUSTIFY_LEFT));
	addProperty("left-margin", prScalar, "int", CAny::createInt(0));
	addProperty("overwrite", prScalar, "bool", CAny::createBool(false));
	addProperty("pixels-above-lines", prScalar, "int", CAny::createInt(0));
	addProperty("pixels-below-lines", prScalar, "int", CAny::createInt(0));
	addProperty("pixels-inside-wrap", prScalar, "int", CAny::createInt(0));
	addProperty("right-margin", prScalar, "int", CAny::createInt(0));
	addProperty("wrap-mode", prScalar, "GtkWrapMode", CAny::createEnum("GtkWrapMode", Gtk::WRAP_NONE));
	addInertProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkTextViewView::getText)).
		setValueSet(sigc::mem_fun(*this, &GtkTextViewView::setText));
}

void GtkTextViewView::initDesign() {
	getObject()->get_buffer()->set_text(getDesignLabel());
}

Any GtkTextViewView::getText(Property * property) {
	return CAny::createString(getObject()->get_buffer()->get_text());
}

void GtkTextViewView::setText(Property * property, Any value) {
	getObject()->get_buffer()->set_text(*value->getString());
}

// GtkIconViewView -------------------------------------------------------------

GtkIconViewView::GtkIconViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("item-orientation", prScalar, "GtkOrientation", CAny::createEnum("GtkOrientation", Gtk::ORIENTATION_VERTICAL));
	addProperty("column-spacing", prScalar, "int", CAny::createInt(6));
	addProperty("columns", prScalar, "int", CAny::createInt(-1));
	addProperty("item-width", prScalar, "int", CAny::createInt(-1));
	addProperty("margin", prScalar, "int", CAny::createInt(6));
	addProperty("row-spacing", prScalar, "int", CAny::createInt(6));
	addProperty("selection-mode", prScalar, "GtkSelectionMode", CAny::createEnum("GtkSelectionMode", Gtk::SELECTION_SINGLE));
	addProperty("spacing", prScalar, "int", CAny::createInt(0));
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkIconViewView::setSampleData));
	addProperty("reorderable", prScalar, "bool", CAny::createBool(false));
	addProperty("item-padding", prScalar, "int", CAny::createInt(6));
}

void GtkIconViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkIconViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::IconView> iconView= getObject();
	if(value->getBool()) {
		PGdkPixbuf pixbuf= Gtk::IconTheme::get_default()->load_icon("gtk-dialog-warning", 32, Gtk::ICON_LOOKUP_FORCE_SIZE);
		Gtk::TreeModel::ColumnRecord columns;
		Gtk::TreeModelColumn<Glib::ustring> textColumn;
		Gtk::TreeModelColumn<PGdkPixbuf> pixbufColumn;
		columns.add(textColumn);
		columns.add(pixbufColumn);
		PGtkListStore model= Gtk::ListStore::create(columns);
		for(int i=0; i<9; ++i) {
			Gtk::TreeRow row= *model->append();
			row[textColumn]= "item" + ToString(i);
			row[pixbufColumn]= pixbuf;
		}
		iconView->set_model(model);
		iconView->set_text_column(textColumn);
		iconView->set_pixbuf_column(pixbufColumn);
	} else {
		iconView->set_model(PGtkListStore());
		iconView->queue_draw();
	}
}

// GtkCellViewView -------------------------------------------------------------

GtkCellViewView::GtkCellViewView() {
	addOrientationProperty();
	addProperty("background-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkCellViewView::setBackgroundSet));
	addProperty("background-rgba", prScalar, "GdkRGBA", CAny::create("GdkRGBA", Black())).
		setFlag(sfReadOnly);
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkCellViewView::setSampleData));
}

void GtkCellViewView::initInstance() {
	GtkWidgetView::initInstance();
	getObject()->set_background_rgba(Black());
	getObject()->property_background_set()= false;
}

void GtkCellViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkCellViewView::setBackgroundSet(Property * property, Any value) {
	bool set= value->getBool();
	getObject()->set_background_rgba(Black());
	getObject()->property_background_set()= set;
	if(set)
		findProperty("background-rgba")->clearFlag(sfReadOnly);
	else
		findProperty("background-rgba")->setFlag(sfReadOnly);
	touch("background-rgba");
}

void GtkCellViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::CellView> cellView= getObject();
	Gtk::TreeModel::ColumnRecord columns;
	Gtk::TreeModelColumn<Glib::ustring> column0;
	Gtk::TreeModelColumn<Glib::ustring> column1;
	Gtk::TreeModelColumn<Glib::ustring> column2;
	columns.add(column0);
	columns.add(column1);
	columns.add(column2);
	PGtkListStore model= Gtk::ListStore::create(columns);
	cellView->clear();
	cellView->set_model(model);
	if(value->getBool()) {
		Gtk::TreeRow row= *model->append();
		row[column0]= "item0";
		row[column1]= "item1";
		row[column2]= "item2";
		cellView->pack_start(column0);
		cellView->pack_start(column1);
		cellView->pack_start(column2);
		cellView->set_displayed_row(model->get_path(row));
	} else {
		cellView->queue_draw();
	}
}

// GtkTreeViewView -------------------------------------------------------------

GtkTreeViewView::GtkTreeViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("enable-search", prScalar, "bool", CAny::createBool(true));
	addInertProperty("fixed-height-mode", prScalar, "bool", CAny::createBool(false));
	addProperty("headers-visible", prScalar, "bool", CAny::createBool(true));
	addProperty("hover-expand", prScalar, "bool", CAny::createBool(false));
	addProperty("hover-selection", prScalar, "bool", CAny::createBool(false));
	addProperty("reorderable", prScalar, "bool", CAny::createBool(false));
	addProperty("rules-hint", prScalar, "bool", CAny::createBool(false));
	addInertProperty("selection-mode", prScalar, "GtkSelectionMode", CAny::createEnum("GtkSelectionMode", Gtk::SELECTION_SINGLE)).
		setValueGet(sigc::mem_fun(*this, &GtkTreeViewView::getSelectionMode)).
		setValueSet(sigc::mem_fun(*this, &GtkTreeViewView::setSelectionMode));
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkTreeViewView::setSampleData));
	addProperty("rubber-banding", prScalar, "bool", CAny::createBool(false));
	addProperty("show-expanders", prScalar, "bool", CAny::createBool(true));
	addProperty("level-indentation", prScalar, "int", CAny::createInt(0));
	addProperty("enable-grid-lines", prScalar, "GtkTreeViewGridLines", CAny::createEnum("GtkTreeViewGridLines", Gtk::TREE_VIEW_GRID_LINES_NONE));
	addProperty("enable-tree-lines", prScalar, "bool", CAny::createBool(false));
	addInertProperty("search-entry", prEntity, "GtkEntry").
		setFlag(sfLinkOnly | sfWeakLink).
		setValueSet(sigc::mem_fun(*this, &GtkTreeViewView::setSearchEntry));
}

void GtkTreeViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkTreeViewView::doneInstance() {
	getObject()->remove_all_columns();
	getObject()->set_model(PGtkTreeModel());
	GtkContainerViewBase::doneInstance();
}

Any GtkTreeViewView::getSelectionMode(Property * property) {
	return CAny::createEnum("GtkSelectionMode", getObject()->get_selection()->get_mode());
}

void GtkTreeViewView::setSelectionMode(Property * property, Any value) {
	getObject()->get_selection()->set_mode((Gtk::SelectionMode)value->getEnum());
}

void GtkTreeViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::TreeView> treeView= getObject();
	treeView->remove_all_columns();
	if(value->getBool()) {
		Gtk::TreeModel::ColumnRecord columns;
		Gtk::TreeModelColumn<Glib::ustring> column0;
		Gtk::TreeModelColumn<Glib::ustring> column1;
		Gtk::TreeModelColumn<Glib::ustring> column2;
		columns.add(column0);
		columns.add(column1);
		columns.add(column2);
		PGtkListStore model= Gtk::ListStore::create(columns);
		for(int i=0; i<3; ++i) {
			Gtk::TreeRow row= *model->append();
			row[column0]= "item" + ToString(i) + "0";
			row[column1]= "item" + ToString(i) + "1";
			row[column2]= "item" + ToString(i) + "2";
		}
		treeView->set_model(model);
		treeView->append_column("Column0", column0);
		treeView->append_column("Column1", column1);
		treeView->append_column("Column2", column2);
	} else {
		treeView->set_model(PGtkTreeModel());
	}
}

void GtkTreeViewView::setSearchEntry(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::Entry> entry= value->getObject<Gtk::Entry>();
	gtk_tree_view_set_search_entry(getObject()->gobj(), entry ? entry->gobj() : NULL);
}

// GtkColorButtonView ----------------------------------------------------------

GtkColorButtonView::GtkColorButtonView() {
	addProperty("alpha", prScalar, "double", CAny::createDouble(1.0)).
		setValueGet(sigc::mem_fun(*this, &GtkColorButtonView::getAlpha)).
		setValueSet(sigc::mem_fun(*this, &GtkColorButtonView::setAlpha));
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("use-alpha", prScalar, "bool", CAny::createBool(false));
	addProperty("color", prScalar, "GdkRGBA", CAny::create("GdkRGBA", Black())).
		setValueGet(sigc::mem_fun(*this, &GtkColorButtonView::getColor)).
		setValueSet(sigc::mem_fun(*this, &GtkColorButtonView::setColor));
}

Any GtkColorButtonView::getAlpha(Property * property) {
	Gdk::RGBA rgba=	getObject()->get_rgba();
	return CAny::createDouble(rgba.get_alpha());
}

void GtkColorButtonView::setAlpha(Property * property, Any value) {
	Gdk::RGBA rgba= getObject()->get_rgba();
	rgba.set_alpha(value->getDouble());
	getObject()->set_rgba(rgba);
}

Any GtkColorButtonView::getColor(Property * property) {
	Gdk::RGBA rgba=	getObject()->get_rgba();
	rgba.set_alpha(1.0);
	return CAny::create<Gdk::RGBA>("GdkRGBA", rgba);
}

void GtkColorButtonView::setColor(Property * property, Any value) {
	Gdk::RGBA rgba= *value->get<Gdk::RGBA>();
	rgba.set_alpha(getObject()->get_rgba().get_alpha());
	getObject()->set_rgba(rgba);
}

// GtkColorSelectionView -------------------------------------------------------

GtkColorSelectionView::GtkColorSelectionView() {
	addProperty("has-opacity-control", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkColorSelectionView::setHasOpacityControl));
	addProperty("has-palette", prScalar, "bool", CAny::createBool(false));
	addProperty("current-alpha", prScalar, "double", CAny::createDouble(1.0)).
		setValueGet(sigc::mem_fun(*this, &GtkColorSelectionView::getCurrentAlpha)).
		setValueSet(sigc::mem_fun(*this, &GtkColorSelectionView::setCurrentAlpha)).
		setFlag(sfReadOnly);
	addProperty("current-color", prScalar, "GdkRGBA", CAny::create("GdkRGBA", Black())).
		setValueGet(sigc::mem_fun(*this, &GtkColorSelectionView::getCurrentColor)).
		setValueSet(sigc::mem_fun(*this, &GtkColorSelectionView::setCurrentColor));
}

void GtkColorSelectionView::setHasOpacityControl(Property * property, Any value) {
	bool has= value->getBool();
	getObject()->set_has_opacity_control(has);
	getObject()->set_current_alpha(65535);
	if(has)
		findProperty("current-alpha")->clearFlag(sfReadOnly);
	else
		findProperty("current-alpha")->setFlag(sfReadOnly);
	touch("current-alpha");
}

Any GtkColorSelectionView::getCurrentAlpha(Property * property) {
	Gdk::RGBA rgba=	getObject()->get_current_rgba();
	return CAny::createDouble(rgba.get_alpha());
}

void GtkColorSelectionView::setCurrentAlpha(Property * property, Any value) {
	Gdk::RGBA rgba= getObject()->get_current_rgba();
	rgba.set_alpha(value->getDouble());
	getObject()->set_current_rgba(rgba);
	getObject()->set_previous_rgba(rgba);
}

Any GtkColorSelectionView::getCurrentColor(Property * property) {
	Gdk::RGBA rgba=	getObject()->get_current_rgba();
	rgba.set_alpha(1.0);
	return CAny::create<Gdk::RGBA>("GdkRGBA", rgba);
}

void GtkColorSelectionView::setCurrentColor(Property * property, Any value) {
	Gdk::RGBA rgba= *value->get<Gdk::RGBA>();
	rgba.set_alpha(getObject()->get_current_rgba().get_alpha());
	getObject()->set_current_rgba(rgba);
	getObject()->set_previous_rgba(rgba);
}

// GtkFileChooserButtonView ----------------------------------------------------

GtkFileChooserButtonView::GtkFileChooserButtonView() {
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("width-chars", prScalar, "int", CAny::createInt(-1));
	addProperty("focus-on-click", prScalar, "bool", CAny::createBool(true));
}

Any GtkFileChooserButtonView::createInstance() {
	Gtk::FileChooserButton * widget= new Gtk::FileChooserButton("", Gtk::FILE_CHOOSER_ACTION_OPEN);
	GObjectClass * oclass= G_OBJECT_GET_CLASS(widget->gobj());
	GParamSpec * property= g_object_class_find_property(oclass, "title");
	Glib::ustring title= G_PARAM_SPEC_STRING(property)->default_value;
	widget->set_title(title);
	return CAny::createObject(getType(), MakeRefPtr(widget));
}

bool GtkFileChooserButtonView::isActiveWidget(Gtk::Widget * widget) {
	return false;
}

// GtkFileChooserWidgetView ----------------------------------------------------

Any GtkFileChooserWidgetView::createInstance() {
	Gtk::Widget * widget= new Gtk::FileChooserWidget(Gtk::FILE_CHOOSER_ACTION_OPEN);
	return CAny::createObject(getType(), MakeRefPtr(widget));
}

// GtkFontButtonView -----------------------------------------------------------

GtkFontButtonView::GtkFontButtonView() {
	addProperty("font-name", prScalar, "string").
		setFlag(sfAutoDefault);
	addProperty("show-size", prScalar, "bool", CAny::createBool(true));
	addProperty("show-style", prScalar, "bool", CAny::createBool(true));
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("use-font", prScalar, "bool", CAny::createBool(false));
	addProperty("use-size", prScalar, "bool", CAny::createBool(false));
}

// GtkFixedView ----------------------------------------------------------------

GtkFixedView::GtkFixedView() {
	addInertProperty("has-window", prScalar, "bool", CAny::createBool(false));
}

void GtkFixedView::initInstance() {
	XYContainerView::initInstance();
	gtk_widget_set_has_window(getObject()->Gtk::Widget::gobj(), TRUE);
}

void GtkFixedView::setupContainer() {
	getContainer()->set(Children());
}

void GtkFixedView::setChildren(Property * property, Any value) {
	property->setInert(value);
	getContainer()->set(makeChildren(value));
}

// GtkLayoutView ---------------------------------------------------------------

GtkLayoutView::GtkLayoutView() {
	addProperty("size", prScalar, "CrowPoint", CAny::createPoint(Point(100, 100))).
		setValueGet(sigc::mem_fun(*this, &GtkLayoutView::getSize)).
		setValueSet(sigc::mem_fun(*this, &GtkLayoutView::setSize));
}

void GtkLayoutView::setupContainer() {
	getContainer()->set(Children());
}

void GtkLayoutView::setChildren(Property * property, Any value) {
	property->setInert(value);
	getContainer()->set(makeChildren(value));
}

Any GtkLayoutView::getSize(Property * property) {
	guint width, height;
	getObject()->get_size(width, height);
	return CAny::createPoint(Point(width, height));
}

void GtkLayoutView::setSize(Property * property, Any value) {
	const Point & point= *value->getPoint();
	getObject()->set_size(point.x, point.y);
}

// GtkScrollbarView ------------------------------------------------------------

GtkScrollbarView::GtkScrollbarView() {
	addCanFocusProperty(false);
}

// GtkEventBoxView -------------------------------------------------------------

GtkEventBoxView::GtkEventBoxView() {
	addAppPaintableProperty();
	addEventProperties();
	addInertProperty("above-child", prScalar, "bool", CAny::createBool(false));
	addInertProperty("visible-window", prScalar, "bool", CAny::createBool(true));
}

// GtkDrawingAreaView ----------------------------------------------------------

GtkDrawingAreaView::GtkDrawingAreaView() {
	addEventProperties();
}

// GtkCalendarView -------------------------------------------------------------

GtkCalendarView::GtkCalendarView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("no-month-change", prScalar, "bool", CAny::createBool(false));
	addProperty("show-day-names", prScalar, "bool", CAny::createBool(true));
	addProperty("show-heading", prScalar, "bool", CAny::createBool(true));
	addProperty("show-week-numbers", prScalar, "bool", CAny::createBool(false));
	addProperty("detail-height-rows", prScalar, "int", CAny::createInt(0));
	addProperty("detail-width-chars", prScalar, "int", CAny::createInt(0));
	addProperty("show-details", prScalar, "bool", CAny::createBool(true));
}

// GtkArrowView ----------------------------------------------------------------

GtkArrowView::GtkArrowView() {
	addProperty("arrow-type", prScalar, "GtkArrowType", CAny::createEnum("GtkArrowType", Gtk::ARROW_RIGHT));
	addProperty("shadow-type", prScalar, "GtkShadowType", CAny::createEnum("GtkShadowType", Gtk::SHADOW_OUT));
}

Any GtkArrowView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr(new Gtk::Arrow(Gtk::ARROW_RIGHT, Gtk::SHADOW_OUT)));
}

// CustomView ------------------------------------------------------------------

CustomView::CustomView() {
	addInertProperty("type", prScalar, "string", CAny::createString(""));
	addInertProperty("meta", prScalar, "string", CAny::createString(""));
	addInertProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
}

Any CustomView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr<Custom>());
}

// DebugButtonView -------------------------------------------------------------

Any DebugButtonView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr<DebugButton>());
}

// DebugWindowView -------------------------------------------------------------

Any DebugWindowView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr<DebugWindow>());
}

// GtkStatusIconView -----------------------------------------------------------

GtkStatusIconView::GtkStatusIconView() {
	addSignalsProperty();
	addInertProperty("visible", prScalar, "bool", CAny::createBool(false));
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addInertProperty("tooltip", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("tooltip-mode", prScalar, "CrowTooltipMode", CAny::createEnum("CrowTooltipMode", TOOLTIP_TEXT));
	addProperty("title", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("name", prScalar, "string", CAny::createString(""));
}

Any GtkStatusIconView::createInstance() {
	Glib::RefPtr<Gtk::StatusIcon> obj= Gtk::StatusIcon::create(PGdkPixbuf());
	obj->set_visible(false);
	return CAny::createObject(getType(), obj);
}

// GtkLinkButtonView -----------------------------------------------------------

GtkLinkButtonView::GtkLinkButtonView() {
	findProperty("relief")->setDefault(CAny::createEnum("GtkReliefStyle", Gtk::RELIEF_NONE));
	addProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("uri", prScalar, "string").
		setValueSet(sigc::mem_fun(*this, &GtkLinkButtonView::setUri)).
		setFlag(sfAutoDefault);
	addProperty("visited", prScalar, "bool", CAny::createBool(false));
}

void GtkLinkButtonView::initDesign() {
	findProperty("label")->setValue(CAny::createString(getDesignLabel()));
}

Any GtkLinkButtonView::createInstance() {
	Gtk::LinkButton * widget= new Gtk::LinkButton("http://www.crowdesigner.org", "");
	return CAny::createObject(getType(), MakeRefPtr(widget));
}

void GtkLinkButtonView::setUri(Property * property, Any value) {
	Glib::RefPtr<Gtk::LinkButton> button= GlibObjectView::getObject<Gtk::LinkButton>();
	bool visited= button->get_visited();
	button->set_uri(*value->getString());
	button->set_visited(visited);
}

// GtkRecentFilterView ---------------------------------------------------------

GtkRecentFilterView::GtkRecentFilterView() {
	addSignalsProperty();
	addProperty("name", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkRecentFilterView::getName)).
		setValueSet(sigc::mem_fun(*this, &GtkRecentFilterView::setName));
	addInertProperty("add-pixbuf-formats", prScalar, "bool", CAny::createBool(false));
	addInertProperty("patterns", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("*.*")
		));
	addInertProperty("mime-types", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
	addInertProperty("applications", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
	addInertProperty("groups", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
	addInertProperty("age", prScalar, "int", CAny::createInt(-1));
}

Any GtkRecentFilterView::createInstance() {
	PGtkRecentFilter filter= Gtk::RecentFilter::create();
	filter->add_pattern("*.*");
	return CAny::createObject(getType(), filter);
}

Any GtkRecentFilterView::getName(Property * property) {
	return CAny::createString(getObject<Gtk::RecentFilter>()->get_name());
}

void GtkRecentFilterView::setName(Property * property, Any value) {
	getObject<Gtk::RecentFilter>()->set_name(*value->getString());
}

// GtkRecentChooserViewBase ----------------------------------------------------

GtkRecentChooserViewBase::GtkRecentChooserViewBase() {
	addProperty("limit", prScalar, "int", CAny::createInt(-1));
	addProperty("local-only", prScalar, "bool", CAny::createBool(true));
	addProperty("select-multiple", prScalar, "bool", CAny::createBool(false));
	addProperty("show-icons", prScalar, "bool", CAny::createBool(true));
	addProperty("show-not-found", prScalar, "bool", CAny::createBool(false));
	addProperty("show-private", prScalar, "bool", CAny::createBool(false));
	addProperty("show-tips", prScalar, "bool", CAny::createBool(false));
	addProperty("sort-type", prScalar, "GtkRecentSortType", CAny::createEnum("GtkRecentSortType", Gtk::RECENT_SORT_NONE));
	addInertProperty("filter", prEntity, "GtkRecentFilter").
		setValueSet(sigc::mem_fun(*this, &GtkRecentChooserViewBase::setFilter));
	addInertProperty("filters", prVector, "GtkRecentFilter").
		setValueSet(sigc::mem_fun(*this, &GtkRecentChooserViewBase::setFilters)).
		setElementInsert(sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GlibObjectView::insertObject));
}

Glib::RefPtr<Gtk::RecentChooser> GtkRecentChooserViewBase::getObject() {
	return Glib::RefPtr<Gtk::RecentChooser>::cast_dynamic(GlibObjectView::getObject());
}

void GtkRecentChooserViewBase::initInstance() {
	GlibObjectView::initInstance();
	initRecentChooserInstance();
}

void GtkRecentChooserViewBase::initRecentChooserInstance() {
	getObject()->set_limit(-1);
	getObject()->set_show_not_found(false);
	getObject()->set_show_tips(false);
}

void GtkRecentChooserViewBase::setFilter(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::RecentFilter> filter= value->getObject<Gtk::RecentFilter>();
	if(filter) {
		clearVectorProperty("filters")->setFlag(sfReadOnly);
		clearFilters();
	} else
		findProperty("filters")->clearFlag(sfReadOnly);
	touch("filters");
	getObject()->property_filter()= filter;
}

void GtkRecentChooserViewBase::setFilters(Property * property, Any value) {
	property->setInert(value);
	clearFilters();
	Glib::RefPtr<Gtk::RecentChooser> chooser= getObject();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PGtkRecentFilter obj= vec[i]->getObject<Gtk::RecentFilter>();
		if(obj)
			chooser->add_filter(obj);
	}
}

void GtkRecentChooserViewBase::clearFilters() {
	Glib::RefPtr<Gtk::RecentChooser> chooser= getObject();
	std::vector<PGtkRecentFilter> list= chooser->list_filters();
	for(int i=0; i<SIGNED(list.size()); ++i)
		if(list[i])
			chooser->remove_filter(list[i]);
}

// GtkRecentChooserWidgetView --------------------------------------------------

void GtkRecentChooserWidgetView::initInstance() {
	GtkContainerViewBase::initInstance();
	initRecentChooserInstance();
}

// GtkRecentChooserDialogView --------------------------------------------------

GtkRecentChooserDialogView::GtkRecentChooserDialogView() {
	findProperty("buttons")->clearFlag(sfReadOnly | sfHidden);
	findProperty("alternative-button-order")->clearFlag(sfReadOnly | sfHidden);
	findProperty("default-response")->clearFlag(sfReadOnly | sfHidden);
	findProperty("border-width")->setFlag(sfReadOnly);
}

Glib::RefPtr<Gtk::RecentChooser> GtkRecentChooserDialogView::getObject() {
	return GlibObjectView::getObject<Gtk::RecentChooserDialog>();
}

void GtkRecentChooserDialogView::initInstance() {
	GtkDialogViewFinal::initInstance();
	initRecentChooserInstance();
}

Any GtkRecentChooserDialogView::createInstance() {
	Gtk::RecentChooserDialog * window= new Gtk::RecentChooserDialog("");
	return CAny::createObject(getType(), MakeRefPtr(window));
}

void GtkRecentChooserDialogView::initDesign() {
	initDesignTitle();
}

// GtkRecentActionView ---------------------------------------------------------

GtkRecentActionView::GtkRecentActionView() {
	findProperty("select-multiple")->setFlag(sfReadOnly);
	findProperty("filters")->setFlag(sfNoEditing);
	addProperty("show-numbers", prScalar, "bool", CAny::createBool(false));
}

Any GtkRecentActionView::createInstance() {
	return CAny::createObject(getType(), Gtk::RecentAction::create(createName()));
}

// GtkSpinnerView --------------------------------------------------------------

GtkSpinnerView::GtkSpinnerView() {
	addInertProperty("active", prScalar, "bool", CAny::createBool(false));
}

}
